@@ -4,16 +4,16 @@
 ///! [`async-datachannel`]: https://crates.io/crates/async-datachannel
 use std::{rc::Rc, task::Poll};
 
-use futures::{stream, StreamExt};
+use anyhow::Context;
+use futures::{
+    channel::mpsc,
+    io::{AsyncRead, AsyncWrite},
+    stream, StreamExt,
+};
 use js_sys::Reflect;
 use log::*;
 use send_wrapper::SendWrapper;
 use serde::{Deserialize, Serialize};
-use tokio::{
-    io::{AsyncRead, AsyncWrite},
-    sync::mpsc,
-};
-use tokio_stream::wrappers::ReceiverStream;
 use wasm_bindgen::{prelude::*, JsCast, JsValue};
 use wasm_bindgen_futures::JsFuture;
 use web_sys::{
@@ -44,8 +44,20 @@ pub enum Message {
     RemoteCandidate(IceCandidate),
 }
 
-/// The opened data channel. This struct implements both [`AsyncRead`] and [`AsyncWrite`]
-/// (tokio-flavoured).
+#[derive(Debug, Clone)]
+pub struct RtcConfig {
+    ice_servers: Vec<String>,
+}
+
+impl RtcConfig {
+    pub fn new<S: AsRef<str>>(ice_servers: &[S]) -> Self {
+        Self {
+            ice_servers: ice_servers.iter().map(|s| s.as_ref().to_string()).collect(),
+        }
+    }
+}
+
+/// The opened data channel. This struct implements both [`AsyncRead`] and [`AsyncWrite`].
 pub struct DataStream {
     /// The actual data channel
     //    inner: Box<RtcDataChannel<DataChannel>>,
@@ -56,7 +68,7 @@ pub struct DataStream {
     // Reference to the PeerConnection to keep around
     //   peer_con: Option<Arc<Mutex<Box<RtcPeerConnection<ConnInternal>>>>>,
     //
-    _on_message: Closure<dyn FnMut(web_sys::MessageEvent)>,
+    _on_message: SendWrapper<Closure<dyn FnMut(web_sys::MessageEvent)>>,
     inner: SendWrapper<Rc<RtcDataChannel>>,
     // Do we need the peer_con?
     //peer_con: RtcPeerConnection,
@@ -65,7 +77,7 @@ pub struct DataStream {
 impl DataStream {
     fn new(inner: RtcDataChannel) -> Self {
         inner.set_binary_type(RtcDataChannelType::Arraybuffer);
-        let (tx, rx_inbound) = mpsc::channel(32);
+        let (mut tx, rx_inbound) = mpsc::channel(32);
         let on_message = Closure::wrap(Box::new(move |ev: web_sys::MessageEvent| {
             let res = match ev.data().dyn_into::<js_sys::ArrayBuffer>() {
                 Ok(data) => {
@@ -77,11 +89,13 @@ impl DataStream {
                     data
                 )),
             };
-            tx.blocking_send(res).expect("FIXME channel died l76");
+            if let Err(e) = tx.try_send(res) {
+                error!("Error sending via channel: {:?}", e);
+            }
         }) as Box<dyn FnMut(web_sys::MessageEvent)>);
         inner.set_onmessage(Some(on_message.as_ref().unchecked_ref()));
         Self {
-            _on_message: on_message,
+            _on_message: SendWrapper::new(on_message),
             inner: SendWrapper::new(Rc::new(inner)),
             buf_inbound: vec![],
             rx_inbound,
@@ -93,35 +107,38 @@ impl AsyncRead for DataStream {
     fn poll_read(
         mut self: std::pin::Pin<&mut Self>,
         cx: &mut std::task::Context<'_>,
-        buf: &mut tokio::io::ReadBuf<'_>,
-    ) -> std::task::Poll<std::io::Result<()>> {
+        buf: &mut [u8],
+    ) -> std::task::Poll<std::io::Result<usize>> {
         if !self.buf_inbound.is_empty() {
-            let space = buf.remaining();
+            let space = buf.len();
             if self.buf_inbound.len() <= space {
-                buf.put_slice(&self.buf_inbound[..]);
+                let len = self.buf_inbound.len();
+                buf[..len].copy_from_slice(&self.buf_inbound[..]);
                 self.buf_inbound.drain(..);
+                Poll::Ready(Ok(len))
             } else {
-                buf.put_slice(&self.buf_inbound[..space]);
+                buf.copy_from_slice(&self.buf_inbound[..space]);
                 self.buf_inbound.drain(..space);
+                Poll::Ready(Ok(space))
             }
-            Poll::Ready(Ok(()))
         } else {
-            match self.as_mut().rx_inbound.poll_recv(cx) {
+            match self.as_mut().rx_inbound.poll_next_unpin(cx) {
                 std::task::Poll::Ready(Some(Ok(x))) => {
-                    let space = buf.remaining();
+                    let space = buf.len();
                     if x.len() <= space {
-                        buf.put_slice(&x[..]);
+                        buf[..x.len()].copy_from_slice(&x[..]);
+                        Poll::Ready(Ok(x.len()))
                     } else {
-                        buf.put_slice(&x[..space]);
+                        buf.copy_from_slice(&x[..space]);
                         self.buf_inbound.extend_from_slice(&x[space..]);
+                        Poll::Ready(Ok(space))
                     }
-                    Poll::Ready(Ok(()))
                 }
                 std::task::Poll::Ready(Some(Err(e))) => Poll::Ready(Err(std::io::Error::new(
                     std::io::ErrorKind::Other,
                     e.to_string(),
                 ))),
-                std::task::Poll::Ready(None) => Poll::Ready(Ok(())),
+                std::task::Poll::Ready(None) => Poll::Ready(Ok(0)),
                 Poll::Pending => Poll::Pending,
             }
         }
@@ -152,7 +169,7 @@ impl AsyncWrite for DataStream {
         Poll::Ready(Ok(()))
     }
 
-    fn poll_shutdown(
+    fn poll_close(
         self: std::pin::Pin<&mut Self>,
         _cx: &mut std::task::Context<'_>,
     ) -> std::task::Poll<Result<(), std::io::Error>> {
@@ -166,40 +183,40 @@ pub struct PeerConnection {
     inner: SendWrapper<Rc<RtcPeerConnection>>,
     sig_tx: mpsc::Sender<Message>,
     sig_rx: mpsc::Receiver<Message>,
-    _on_ice_candidate: Closure<dyn FnMut(RtcPeerConnectionIceEvent)>,
+    _on_ice_candidate: SendWrapper<Closure<dyn FnMut(RtcPeerConnectionIceEvent)>>,
 }
 
 impl PeerConnection {
     /// Create a new [`PeerConnection`] to be used for either dialing or accepting an inbound
     /// connection. The channel tuple is used to interface with an external signalling system.
     pub fn new(
-        turn: Vec<String>,
+        config: &RtcConfig,
         (sig_tx, sig_rx): (mpsc::Sender<Message>, mpsc::Receiver<Message>),
     ) -> anyhow::Result<Self> {
-        let mut config = RtcConfiguration::new();
+        let mut rtc_config = RtcConfiguration::new();
 
         let ice_servers = js_sys::Array::new();
-        for s in turn {
+        for s in &config.ice_servers {
             // TODO: handle stun?
             let mut stun_server = RtcIceServer::new();
             let stun_servers = js_sys::Array::new();
-            stun_servers.push(&JsValue::from(&s));
+            stun_servers.push(&JsValue::from(s));
             stun_server.urls(&stun_servers);
             ice_servers.push(&JsValue::from(&stun_server));
         }
-        config.ice_servers(&ice_servers);
+        rtc_config.ice_servers(&ice_servers);
 
-        let inner = RtcPeerConnection::new_with_configuration(&config)
-            .map_err(|e| anyhow::anyhow!("FIXME creating peer connection {:?}", e.as_string()))?;
+        let inner = RtcPeerConnection::new_with_configuration(&rtc_config)
+            .map_err(|e| anyhow::anyhow!("Error creating peer connection {:?}", e.as_string()))?;
 
-        let sig_tx_c = sig_tx.clone();
+        let mut sig_tx_c = sig_tx.clone();
         let on_ice_candidate = Closure::wrap(Box::new(move |ev: RtcPeerConnectionIceEvent| {
             if let Some(candidate) = ev.candidate() {
-                if let Err(e) = sig_tx_c.blocking_send(Message::RemoteCandidate(IceCandidate {
+                if let Err(e) = sig_tx_c.try_send(Message::RemoteCandidate(IceCandidate {
                     candidate: candidate.candidate(),
                     mid: candidate.sdp_mid().unwrap_or_else(|| "".to_string()),
                 })) {
-                    todo!("Sending failed {:?}", e);
+                    error!("Sending via sig_tx failed {:?}", e);
                 }
             }
         })
@@ -210,7 +227,7 @@ impl PeerConnection {
             inner: SendWrapper::new(Rc::new(inner)),
             sig_rx,
             sig_tx,
-            _on_ice_candidate: on_ice_candidate,
+            _on_ice_candidate: SendWrapper::new(on_ice_candidate),
         })
     }
 
@@ -224,33 +241,30 @@ impl PeerConnection {
         let Self {
             inner,
             sig_rx,
-            sig_tx,
+            mut sig_tx,
             ..
         } = self;
         enum Either<A, B> {
             Left(A),
             Right(B),
         }
-        let (tx_open, mut rx_open) = mpsc::channel(1);
-        let (tx_chan, rx_chan) = mpsc::channel(1);
+        let (mut tx_open, mut rx_open) = mpsc::channel(1);
+        let (mut tx_chan, rx_chan) = mpsc::channel(1);
 
         let on_open = Closure::wrap(Box::new(move || {
             trace!("Inbound data channel opened");
-            tx_open.blocking_send(()).expect("channel diend l226");
+            tx_open.try_send(()).expect("channel diend l226");
         }) as Box<dyn FnMut()>);
         let on_data_channel = Closure::wrap(Box::new(move |ev: RtcDataChannelEvent| {
             trace!("Inbound connection attempt");
             let channel = ev.channel();
             channel.set_onopen(Some(on_open.as_ref().unchecked_ref()));
-            if let Err(e) = tx_chan.blocking_send(channel) {
-                todo!("err {:?}", e);
+            if let Err(e) = tx_chan.try_send(channel) {
+                error!("err sending via channel {:?}", e);
             }
         }) as Box<dyn FnMut(RtcDataChannelEvent)>);
         inner.set_ondatachannel(Some(on_data_channel.as_ref().unchecked_ref()));
-        let mut s = stream::select(
-            ReceiverStream::new(sig_rx).map(Either::Left),
-            ReceiverStream::new(rx_chan).map(Either::Right),
-        );
+        let mut s = stream::select(sig_rx.map(Either::Left), rx_chan.map(Either::Right));
 
         while let Some(m) = s.next().await {
             match m {
@@ -262,29 +276,37 @@ impl PeerConnection {
                             description.sdp(&desc.sdp);
                             JsFuture::from(inner.set_remote_description(&description))
                                 .await
-                                .expect("FIXME set remote descri l249");
+                                .map_err(|e| {
+                                    anyhow::anyhow!("Error setting remote description: {:?}", e)
+                                })?;
 
                             let answer = JsFuture::from(inner.create_answer())
                                 .await
-                                .expect("FIXME create answer l252");
+                                .map_err(|e| anyhow::anyhow!("Error creating answer: {:?}", e))?;
                             let answer_sdp = Reflect::get(&answer, &JsValue::from_str("sdp"))
-                                .expect("FIXME get sdp from answer l254")
+                                .map_err(|e| {
+                                    anyhow::anyhow!("Error extracting sdp from answer: {:?}", e)
+                                })?
                                 .as_string()
                                 .unwrap();
                             let mut answer_obj = RtcSessionDescriptionInit::new(RtcSdpType::Answer);
                             answer_obj.sdp(&answer_sdp);
                             JsFuture::from(inner.set_local_description(&answer_obj))
                                 .await
-                                .expect("FIXME set local desc");
+                                .map_err(|e| {
+                                    anyhow::anyhow!("Error setting local description: {:?}", e)
+                                })?;
 
-                            sig_tx
-                                .send(Message::RemoteDescription(SessionDescription {
+                            if let Err(e) =
+                                sig_tx.try_send(Message::RemoteDescription(SessionDescription {
                                     sdp_type: "answer".into(),
                                     sdp: answer_sdp,
                                 }))
-                                .await
-                                .expect("FIXME channel died l264");
-                            trace!("Sent answer to remote");
+                            {
+                                error!("Error sending answer via channel: {:?}", e);
+                            } else {
+                                trace!("Sent answer to remote");
+                            }
                         }
                     }
                     Message::RemoteCandidate(c) => {
@@ -294,7 +316,7 @@ impl PeerConnection {
                             inner.add_ice_candidate_with_opt_rtc_ice_candidate_init(Some(&cand)),
                         )
                         .await
-                        .expect("FIXME add ice candidate");
+                        .map_err(|e| anyhow::anyhow!("Error adding ice candidate: {:?}", e))?;
                     }
                 },
                 Either::Right(dc) => {
@@ -302,7 +324,7 @@ impl PeerConnection {
                     inner.set_onicecandidate(None);
                     inner.set_ondatachannel(None);
 
-                    rx_open.recv().await.expect("FIXME channel died l287");
+                    rx_open.next().await.context("Waiting for open")?;
                     dc.set_onopen(None);
                     return Ok(DataStream::new(dc));
                 }
@@ -320,7 +342,7 @@ impl PeerConnection {
     /// set_remote_description(&answer)
     pub async fn dial(self, label: &str) -> anyhow::Result<DataStream> {
         let Self {
-            sig_tx,
+            mut sig_tx,
             inner,
             sig_rx,
             ..
@@ -330,23 +352,22 @@ impl PeerConnection {
             Left(A),
             Right(B),
         }
-        let (tx_open, rx_open) = mpsc::channel::<()>(1);
+        let (mut tx_open, rx_open) = mpsc::channel::<()>(1);
 
         let on_open = Closure::wrap(Box::new(move || {
             trace!("Outbound Datachannel opened");
-            tx_open.blocking_send(()).expect("FIXME channel died l318");
+            if let Err(e) = tx_open.try_send(()) {
+                error!("Error sending opening event: {:?}", e);
+            }
         }) as Box<dyn FnMut()>);
         dc.set_onopen(Some(on_open.as_ref().unchecked_ref()));
-        let mut s = stream::select(
-            ReceiverStream::new(sig_rx).map(Either::Left),
-            ReceiverStream::new(rx_open).map(Either::Right),
-        );
+        let mut s = stream::select(sig_rx.map(Either::Left), rx_open.map(Either::Right));
 
         let offer = JsFuture::from(inner.create_offer())
             .await
-            .expect("FIXME creating offer");
+            .map_err(|e| anyhow::anyhow!("Error creating offer: {:?}", e))?;
         let offer_sdp = Reflect::get(&offer, &JsValue::from_str("sdp"))
-            .expect("FIXME extracting sdp from offer")
+            .map_err(|e| anyhow::anyhow!("Error extracting sdp from offer: {:?}", e))?
             .as_string()
             .unwrap();
 
@@ -355,14 +376,13 @@ impl PeerConnection {
         let sld_promise = inner.set_local_description(&offer_obj);
         JsFuture::from(sld_promise)
             .await
-            .expect("FIXME set local desc l335");
+            .map_err(|e| anyhow::anyhow!("Error setting local description: {:?}", e))?;
         sig_tx
-            .send(Message::RemoteDescription(SessionDescription {
+            .try_send(Message::RemoteDescription(SessionDescription {
                 sdp_type: "offer".into(),
                 sdp: offer_sdp,
             }))
-            .await
-            .expect("FIXME channel died l342");
+            .context("Signaling channel closed")?;
 
         while let Some(m) = s.next().await {
             match m {
@@ -371,12 +391,14 @@ impl PeerConnection {
                         if desc.sdp_type == "answer" {
                             let mut description = RtcSessionDescriptionInit::new(
                                 RtcSdpType::from_js_value(&JsValue::from_str(&desc.sdp_type))
-                                    .expect("FIXME create rtc type l351"),
+                                    .context("Error creating rtc session description")?,
                             );
                             description.sdp(&desc.sdp);
                             JsFuture::from(inner.set_remote_description(&description))
                                 .await
-                                .expect("FIXME set remote desc l356");
+                                .map_err(|e| {
+                                    anyhow::anyhow!("Error setting remote description: {:?}", e)
+                                })?;
                         }
                     }
                     Message::RemoteCandidate(c) => {
@@ -386,7 +408,7 @@ impl PeerConnection {
                             inner.add_ice_candidate_with_opt_rtc_ice_candidate_init(Some(&cand)),
                         )
                         .await
-                        .expect("FIXME add ice cand l366");
+                        .map_err(|e| anyhow::anyhow!("Error adding ice candidate: {:?}", e))?;
                     }
                 },
                 Either::Right(_) => {
